@@ -37,6 +37,7 @@ ephy_history_service_initialize_urls_table (EphyHistoryService *self)
                                   "host INTEGER NOT NULL REFERENCES hosts(id) ON DELETE CASCADE,"
                                   "url LONGVARCAR,"
                                   "title LONGVARCAR,"
+                                  "sync_id LONGVARCAR,"
                                   "visit_count INTEGER DEFAULT 0 NOT NULL,"
                                   "typed_count INTEGER DEFAULT 0 NOT NULL,"
                                   "last_visit_time INTEGER,"
@@ -67,11 +68,11 @@ ephy_history_service_get_url_row (EphyHistoryService *self, const char *url_stri
 
   if (url != NULL && url->id != -1) {
     statement = ephy_sqlite_connection_create_statement (self->history_database,
-                                                         "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time FROM urls "
+                                                         "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time, sync_id FROM urls "
                                                          "WHERE id=?", &error);
   } else {
     statement = ephy_sqlite_connection_create_statement (self->history_database,
-                                                         "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time FROM urls "
+                                                         "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time, sync_id FROM urls "
                                                          "WHERE url=?", &error);
   }
 
@@ -116,6 +117,7 @@ ephy_history_service_get_url_row (EphyHistoryService *self, const char *url_stri
   url->last_visit_time = ephy_sqlite_statement_get_column_as_int64 (statement, 5);
   url->hidden = ephy_sqlite_statement_get_column_as_int (statement, 6);
   url->thumbnail_time = ephy_sqlite_statement_get_column_as_int64 (statement, 7);
+  url->sync_id = g_strdup (ephy_sqlite_statement_get_column_as_string (statement, 8));
 
   g_object_unref (statement);
   return url;
@@ -131,8 +133,8 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
   g_assert (self->history_database != NULL);
 
   statement = ephy_sqlite_connection_create_statement (self->history_database,
-                                                       "INSERT INTO urls (url, title, visit_count, typed_count, last_visit_time, host) "
-                                                       " VALUES (?, ?, ?, ?, ?, ?)", &error);
+                                                       "INSERT INTO urls (url, title, visit_count, typed_count, last_visit_time, host, sync_id) "
+                                                       " VALUES (?, ?, ?, ?, ?, ?, ?)", &error);
   if (error) {
     g_warning ("Could not build urls table addition statement: %s", error->message);
     g_error_free (error);
@@ -144,7 +146,8 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
       ephy_sqlite_statement_bind_int (statement, 2, url->visit_count, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 3, url->typed_count, &error) == FALSE ||
       ephy_sqlite_statement_bind_int64 (statement, 4, url->last_visit_time, &error) == FALSE ||
-      ephy_sqlite_statement_bind_int (statement, 5, url->host->id, &error) == FALSE) {
+      ephy_sqlite_statement_bind_int (statement, 5, url->host->id, &error) == FALSE ||
+      ephy_sqlite_statement_bind_string (statement, 6, url->sync_id, &error) == FALSE) {
     g_warning ("Could not insert URL into urls table: %s", error->message);
     g_error_free (error);
     g_object_unref (statement);
@@ -172,7 +175,7 @@ ephy_history_service_update_url_row (EphyHistoryService *self, EphyHistoryURL *u
   g_assert (self->history_database != NULL);
 
   statement = ephy_sqlite_connection_create_statement (self->history_database,
-                                                       "UPDATE urls SET title=?, visit_count=?, typed_count=?, last_visit_time=?, hidden_from_overview=?, thumbnail_update_time=? "
+                                                       "UPDATE urls SET title=?, visit_count=?, typed_count=?, last_visit_time=?, hidden_from_overview=?, thumbnail_update_time=?, sync_id=? "
                                                        "WHERE id=?", &error);
   if (error) {
     g_warning ("Could not build urls table modification statement: %s", error->message);
@@ -186,7 +189,8 @@ ephy_history_service_update_url_row (EphyHistoryService *self, EphyHistoryURL *u
       ephy_sqlite_statement_bind_int64 (statement, 3, url->last_visit_time, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 4, url->hidden, &error) == FALSE ||
       ephy_sqlite_statement_bind_int64 (statement, 5, url->thumbnail_time, &error) == FALSE ||
-      ephy_sqlite_statement_bind_int (statement, 6, url->id, &error) == FALSE) {
+      ephy_sqlite_statement_bind_string (statement, 6, url->sync_id, &error) == FALSE ||
+      ephy_sqlite_statement_bind_int (statement, 7, url->id, &error) == FALSE) {
     g_warning ("Could not modify URL in urls table: %s", error->message);
     g_error_free (error);
     g_object_unref (statement);
@@ -215,6 +219,7 @@ create_url_from_statement (EphySQLiteStatement *statement)
   url->hidden = ephy_sqlite_statement_get_column_as_int (statement, 6);
   url->thumbnail_time = ephy_sqlite_statement_get_column_as_int64 (statement, 7);
   url->host->id = ephy_sqlite_statement_get_column_as_int (statement, 8);
+  url->sync_id = g_strdup (ephy_sqlite_statement_get_column_as_string (statement, 9));
 
   return url;
 }
@@ -237,7 +242,8 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
                                "urls.last_visit_time, "
                                "urls.hidden_from_overview, "
                                "urls.thumbnail_update_time, "
-                               "urls.host "
+                               "urls.host, "
+                               "urls.sync_id "
                                "FROM "
                                "urls ";
 
@@ -317,7 +323,7 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   }
 
   if (query->from > 0) {
-    if (ephy_sqlite_statement_bind_int (statement, i++, (int)query->from, &error) == FALSE) {
+    if (ephy_sqlite_statement_bind_int64 (statement, i++, query->from, &error) == FALSE) {
       g_warning ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
       g_object_unref (statement);
@@ -325,7 +331,7 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
     }
   }
   if (query->to > 0) {
-    if (ephy_sqlite_statement_bind_int (statement, i++, (int)query->to, &error) == FALSE) {
+    if (ephy_sqlite_statement_bind_int64 (statement, i++, query->to, &error) == FALSE) {
       g_warning ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
       g_object_unref (statement);
